USE [igops]
GO

DROP VIEW IF EXISTS cpol_overview_v
GO

DROP VIEW IF EXISTS access_request_deleted_membership_v
GO

DROP VIEW IF EXISTS deleted_entities_v
GO

DROP VIEW IF EXISTS deleted_brole_entities_v
GO

CREATE VIEW cpol_overview_v AS 
	SELECT 
		cpol.ID,
		cpol.unique_id,
		cpol.NAME,
		cpol.DESCRIPTION,
		cpol.VALIDITY_PERIOD,
		cpol.status,
		cpol.CP_TOTAL_VIOLATION
	FROM CERTIFICATION_POLICY cpol
	WHERE cpol.IS_ACTIVE = '1'
GO

CREATE VIEW deleted_entities_v AS

SELECT suser.id,
suser.unique_user_id AS unique_id,
suser.display_name,
suser.effective,
suser.deleted,
'USER' AS type,
COALESCE(suser.last_modified, ( SELECT min(sversion."timestamp") AS min
	FROM sversion
	JOIN data_producer dp ON sversion.data_producer = dp.id
	WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > suser.sversion)) AS "timestamp"
FROM suser
LEFT JOIN ( SELECT suser_1.id AS suser,
				   suser_1.display_name,
				   string_agg(suser_email_1.email, ' | ') WITHIN GROUP (ORDER BY suser_email_1.email) AS email
			  FROM suser suser_1
     JOIN suser_email suser_email_1 ON suser_1.id = suser_email_1.suser AND suser_1.effective = 1 AND suser_1.deleted = 0
  GROUP BY suser_1.id, suser_1.display_name) suser_email ON suser.id = suser_email.suser
WHERE suser.deleted = 1 AND suser.effective = 1

UNION ALL

SELECT sg.id,
	   sg.unique_group_id AS unique_id,
	   sg.name AS display_name,
	   sg.effective,
	   sg.deleted,
	   'GROUP' AS type,
	   COALESCE(sg.last_modified, ( SELECT min(sversion."timestamp") AS min
  FROM sversion
  JOIN data_producer dp ON sversion.data_producer = dp.id
 WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sg.sversion)) AS "timestamp"
FROM sgroup sg
LEFT JOIN sgroup_search sgs ON sgs.entity = sg.id
WHERE sg.effective = 1 AND sg.deleted = 1

GO

CREATE VIEW access_request_deleted_membership_v AS

SELECT air.policy_id,
	arp.name,
	arp.deleted AS policy_is_deleted,
	air.requester_type,
	'ALLOWED MEMBERSHIP' AS designation,
	air.requester_unique_id,
	air.requester_name,
	air.id,
	de.deleted is_deleted
	FROM allowed_item_requester air
	LEFT JOIN access_req_policy arp ON arp.id = air.policy_id
	LEFT JOIN deleted_entities_v de ON de.unique_id = air.requester_unique_id
	UNION
	SELECT dir.policy_id,
	arp.name,
	arp.deleted AS policy_is_deleted,
	dir.requester_type,
	'DISALLOWED MEMBERSHIP' AS designation,
	dir.requester_unique_id,
	dir.requester_name,
	dir.id,
	de.deleted
	FROM disallowed_item_requester dir
	LEFT JOIN access_req_policy arp ON arp.id = dir.policy_id
	LEFT JOIN deleted_entities_v de ON de.unique_id = dir.requester_unique_id

GO

CREATE VIEW deleted_brole_entities_v AS

SELECT brole.id AS brole_id, 
		    brole.unique_id,
		    brole.name,
		    brole.publish_time,
		    brole.approval_state,
			CASE
			    WHEN included_members.unique_user_id IS NOT NULL THEN 'USER'
			    WHEN included_groups.unique_group_id IS NOT NULL THEN 'GROUP'
			    ELSE NULL
			END AS type,
			CASE
			    WHEN included_members.unique_user_id IS NOT NULL THEN 'INCLUDED_MEMBER'
			    WHEN included_groups.unique_group_id IS NOT NULL THEN 'INCLUDED_GROUP'
			    ELSE NULL
			END AS designation,
		    brole_mem.unique_member_id AS item_id,
		    COALESCE(included_members.display_name, included_groups.name) AS item_name,
		    CASE
			WHEN included_members.unique_user_id IS NOT NULL THEN 
				COALESCE(included_members.last_modified, ( SELECT min(sversion."timestamp") AS min
					FROM sversion
					JOIN data_producer dp ON sversion.data_producer = dp.id
					WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > included_members.sversion))
			WHEN included_groups.unique_group_id IS NOT NULL THEN
				COALESCE(included_groups.last_modified, ( SELECT min(sversion."timestamp") AS min
					FROM sversion
					JOIN data_producer dp ON sversion.data_producer = dp.id
					WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > included_groups.sversion))
		     END AS deletion_time
		   FROM business_role brole
		     JOIN business_role_incl_memb brole_mem ON brole.id = brole_mem.business_role_id
		     LEFT JOIN suser included_members ON brole_mem.unique_member_id = included_members.unique_user_id AND included_members.effective = 1 AND included_members.deleted = 1
		     LEFT JOIN sgroup included_groups ON brole_mem.unique_member_id = included_groups.unique_group_id AND included_groups.effective = 1 AND included_groups.deleted = 1
		   WHERE brole.approval_state != 'ARCHIVE' AND (included_members.unique_user_id IS NOT NULL OR included_groups.unique_group_id IS NOT NULL)
		UNION ALL
		 SELECT brole.id AS brole_id,
		    brole.unique_id,
		    brole.name,
		    brole.publish_time,
		    brole.approval_state,
			CASE
			    WHEN excl_members.unique_user_id IS NOT NULL THEN 'USER'
			    WHEN excl_groups.unique_group_id IS NOT NULL THEN 'GROUP'
			    ELSE NULL
			END AS type,
			CASE
			    WHEN excl_members.unique_user_id IS NOT NULL THEN 'EXCLUDED_MEMBER'
			    WHEN excl_groups.unique_group_id IS NOT NULL THEN 'EXCLUDED_GROUP'
			    ELSE NULL
			END AS designation,
		    brole_excl_mem.unique_member_id AS item_id,
		    COALESCE(excl_members.display_name, excl_groups.name) AS item_name,
		    CASE
			WHEN excl_members.unique_user_id IS NOT NULL THEN 
				COALESCE(excl_members.last_modified, ( SELECT min(sversion."timestamp") AS min
					FROM sversion
					JOIN data_producer dp ON sversion.data_producer = dp.id
					WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > excl_members.sversion))
			WHEN excl_groups.unique_group_id IS NOT NULL THEN
				COALESCE(excl_groups.last_modified, ( SELECT min(sversion."timestamp") AS min
					FROM sversion
					JOIN data_producer dp ON sversion.data_producer = dp.id
					WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > excl_groups.sversion))
		     END AS deletion_time
		   FROM business_role brole
		     JOIN business_role_excl_memb brole_excl_mem ON brole.id = brole_excl_mem.business_role_id
		     LEFT JOIN suser excl_members ON brole_excl_mem.unique_member_id = excl_members.unique_user_id AND excl_members.effective = 1 AND excl_members.deleted = 1
		     LEFT JOIN sgroup excl_groups ON brole_excl_mem.unique_member_id = excl_groups.unique_group_id AND excl_groups.effective = 1 AND excl_groups.deleted = 1
		   WHERE brole.approval_state != 'ARCHIVE' AND (excl_members.unique_user_id IS NOT NULL OR excl_groups.unique_group_id IS NOT NULL)
		  UNION ALL

		   --BROLE OWNERS
		   SELECT  brole.id AS brole_id,
	     brole.unique_id,
	     brole.name,
	     brole.publish_time,
	     brole.approval_state,
		CASE
		    WHEN su.unique_user_id IS NOT NULL THEN 'USER'
		    WHEN sg.unique_group_id IS NOT NULL THEN 'GROUP'
		    ELSE NULL
		END AS type,
	     'OWNER' AS designation,
	     COALESCE(su.unique_user_id, sg.unique_group_id) AS item_id,
	     COALESCE(su.display_name, sg.name) AS item_name,
	     CASE
		WHEN su.unique_user_id IS NOT NULL THEN 
			COALESCE(su.last_modified, ( SELECT min(sversion."timestamp") AS min
				FROM sversion
				JOIN data_producer dp ON sversion.data_producer = dp.id
				WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > su.sversion))
		WHEN sg.unique_group_id IS NOT NULL THEN
			COALESCE(sg.last_modified, ( SELECT min(sversion."timestamp") AS min
				FROM sversion
				JOIN data_producer dp ON sversion.data_producer = dp.id
				WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sg.sversion))
	    END AS deletion_time
	   FROM business_role brole
	   JOIN business_role_owner bro ON bro.business_role_id = brole.id
	   LEFT JOIN suser su ON bro.unique_owner_id = su.unique_user_id AND su.effective = 1 AND su.deleted = 1
	   LEFT JOIN sgroup sg ON bro.unique_owner_id = sg.unique_group_id AND sg.effective = 1 AND sg.deleted = 1
	   WHERE brole.approval_state != 'ARCHIVE' AND (su.unique_user_id IS NOT NULL OR sg.unique_group_id IS NOT NULL)
	   UNION ALL 

	   --BROLE MANAGERS
	   SELECT brole.id AS business_role_id,
		brole.unique_id,
		brole.name,
		brole.publish_time,
		brole.approval_state,
		CASE
		    WHEN su.unique_user_id IS NOT NULL THEN 'USER'
		    WHEN sg.unique_group_id IS NOT NULL THEN 'GROUP'
		    ELSE NULL
		END AS type,
		'MANAGER' AS designation,
		COALESCE(su.unique_user_id, sg.unique_group_id) AS item_id,
		COALESCE(su.display_name, sg.name) AS item_name,
		CASE
			WHEN su.unique_user_id IS NOT NULL THEN 
				COALESCE(su.last_modified, ( SELECT min(sversion."timestamp") AS min
					FROM sversion
					JOIN data_producer dp ON sversion.data_producer = dp.id
					WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > su.sversion))
			WHEN sg.unique_group_id IS NOT NULL THEN
				COALESCE(sg.last_modified, ( SELECT min(sversion."timestamp") AS min
					FROM sversion
					JOIN data_producer dp ON sversion.data_producer = dp.id
					WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sg.sversion))
		END AS deletion_time
	FROM business_role brole
	JOIN auth_scope auth_scope ON auth_scope.unique_business_role_id = brole.unique_id
	JOIN auth_role_mapping ON auth_role_mapping.auth_scope = auth_scope.id AND auth_role_mapping.auth_role = 27
	LEFT JOIN suser su ON auth_role_mapping.unique_user_id = su.unique_user_id AND su.deleted = 1 AND su.effective = 1
	LEFT JOIN sgroup sg ON auth_role_mapping.unique_group_id = sg.unique_group_id AND sg.deleted = 1 AND sg.effective = 1
	WHERE brole.approval_state != 'ARCHIVE' AND (su.unique_user_id IS NOT NULL OR sg.unique_group_id IS NOT NULL)

	UNION ALL

	--BROLE FULFILLERS
	
	SELECT brole.id AS business_role_id,
	    brole.unique_id,
	    brole.name,
	    brole.publish_time,
	    brole.approval_state,
	    CASE
		WHEN su.unique_user_id IS NOT NULL THEN 'USER'
		WHEN sg.unique_group_id IS NOT NULL THEN 'GROUP'
		ELSE NULL
	    END AS type,
	    'FULFILLER' AS designation,
	    COALESCE(su.unique_user_id, sg.unique_group_id) AS item_id,
	    COALESCE(su.display_name, sg.name) AS item_name,
	    CASE
		WHEN su.unique_user_id IS NOT NULL THEN 
			COALESCE(su.last_modified, ( SELECT min(sversion."timestamp") AS min
				FROM sversion
				JOIN data_producer dp ON sversion.data_producer = dp.id
				WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > su.sversion))
		WHEN sg.unique_group_id IS NOT NULL THEN
			COALESCE(sg.last_modified, ( SELECT min(sversion."timestamp") AS min
				FROM sversion
				JOIN data_producer dp ON sversion.data_producer = dp.id
				WHERE dp.data_producer_type = 'UNIFIER' AND sversion.id > sg.sversion))
	    END AS deletion_time
	   FROM business_role brole
	   JOIN auth_scope ON auth_scope.unique_business_role_id = brole.unique_id
	   JOIN auth_role_mapping ar_map ON ar_map.auth_scope = auth_scope.id AND ar_map.auth_role = 29
	   LEFT JOIN suser su ON ar_map.unique_user_id = su.unique_user_id AND su.display_name IS NOT NULL AND su.deleted = 1 AND su.effective = 1
	   LEFT JOIN sgroup sg ON ar_map.unique_group_id = sg.unique_group_id AND sg.name IS NOT NULL AND sg.deleted = 1 AND sg.effective = 1
	   WHERE brole.approval_state != 'ARCHIVE' AND (su.unique_user_id IS NOT NULL OR sg.unique_group_id IS NOT NULL);
GO

GRANT SELECT ON cpol_overview_v TO ig_report_role
GO

GRANT SELECT ON access_request_deleted_membership_v TO ig_report_role
GO

GRANT SELECT ON deleted_entities_v TO ig_report_role
GO

GRANT SELECT ON deleted_brole_entities_v TO ig_report_role
GO